"""
Functions to take parsed HTML (represented as beautiful soup objects) and
return rows of (commenter, comment).
"""

from HTMLParser import HTMLParseError
from functools import wraps
import bs4
from doxcommentextractor.types import Comment

def extract(post):
    """Extracts comments from the HTML of a Post object.

    Args:
        post -- a doxcommentextractor.types.Post object

    Returns:
        A pair of values.  The first value is a boolean description of whether
        the HTML of the post could be successfuly parsed.

        If yes, the first value will be True, and the second value will
        be an array of doxcommentextractor.types.Comment objects, describing
        the comments made on that posting.

        If no, the first value will be False, and the second value will be
        an expection object describing the parsing error.
    """
    if post.network == "facebook":
        return facebook(post.text)
    elif post.network == "instagram":
        return instagram(post.text)
    else:
        msg = "Network {} is not a OSN we know how to parse comments for".format(post.network)
        return False, Exception(msg)

def parse_html5(func):
    """Decorator function for receiving an HTML string, and returning
    either a BeautifulSoup parsed representation of the HTML, or
    an error describing what went wrong.

    Args:
        html -- A unicode string, describing a HTML document.

    Returns:
        Calls the wrapped function with a BeautifulSoup instance if the string
        could be parsed as HTML, or otherwise returns
        (False, HTMLParser.HTMLParseError) if there was an error parsing
        the string.
    """
    @wraps(func)
    def func_wrapper(html):
        try:
            document = bs4.BeautifulSoup(html, 'html.parser')
            return func(document)
        except HTMLParseError, err:
            return False, err
    return func_wrapper

@parse_html5
def instagram(html):
    """Returns a list of zero or more comments found in the HTML describing
    an instagram posting.

    Args:
        html -- a unicode string, representing a instagram posting.

    Returns:
        A pair of values, with the first being a boolean description of
        whether the given HTML string could be parsed into a document.  If
        the first value is False, then the second value is an exception,
        describing the parsing error.

        If the first value is true, the second value is a list of zero or more
        comments (named tuples, each with a "comment" and a "commenter" key).
    """
    comments = []
    comment_elms = html.select("article div ul li")
    for comment_elm in comment_elms:
        commenter_elm = comment_elm.find('a')
        if commenter_elm is None:
            continue
        commenter_name = commenter_elm.get_text()
        comment_html = comment_elm.find('span').get_text()
        comments.append(Comment(commenter_name, comment_html))
    return True, comments

@parse_html5
def facebook(html):
    """Returns a list of zero or more comments found in the HTML describing
    an facebook posting.

    Args:
        html -- a unicode string, representing a facebook posting.

    Returns:
        A pair of values, with the first being a boolean description of
        whether the given HTML string could be parsed into a document.  If
        the first value is False, then the second value is a string, describing
        the parsing error.

        If the first value is true, the second value is a list of zero or more
        comments (named tuples, each with a "comment" and a "commenter" key).
    """
    comments = []
    comment_elms = html.select(".UFIComment")
    for comment_elm in comment_elms:
        commenter_elm = comment_elm.select(".UFICommentActorName")
        if commenter_elm is None:
            continue
        commenter_name = commenter_elm[0].get_text()
        comment_html = comment_elm.select(".UFICommentBody")[0].get_text()
        comments.append(Comment(commenter_name, comment_html))
    return True, comments
