"""
Functions for creating database operations to find OSN postings to extract
comments from (and to measure abuse in).
"""

import itertools
import psycopg2
from doxcommentextractor.types import Post, Mention, MentionLabel

def fetch_mentions_to_evaluate(psql, limit=100):
    """Fetches a list of twitter mentions that have not yet been scored for
    whether they're abusive or not.

    This function currently only returns Twitter posts.

    Args:
        psql -- A psycopg2.connection instance.

    Keyword Args:
        limit -- An integer, describing the maximum number of posts that
                 should be returned.

    Returns:
        A pair of objects, the first being a boolean description of whether
        we were successfully able to connect to the database and perform the
        expected queries.

        If the first value is False, the second value will be an exception
        object, describing the database error.

        If the first value is True, the second value will be a list of zero
        or more Mention values, describing twitter mentions that haven't had
        their content evaluated for abuse yet.
    """
    cursor = psql.cursor()
    select_query = """
        SELECT
            m.id,
            m.commenter,
            m.text
        FROM
            mentions AS m
        WHERE
            m.is_abusive IS NULL
        LIMIT
            %s;
    """

    try:
        cursor.execute(select_query, (int(limit),))
        return True, [Mention(*row) for row in cursor.fetchall()]
    except ValueError, err:
        return False, Exception("Invalid integer provided as a limit.")
    except psycopg2.Error, err:
        return False, err

def record_if_mentions_are_abusive(psql, mention_labels):
    """Updates the database records for each provided mention, recording whether
    they're abusive or not.

    Args:
        psql           -- A psycopg2.connection instance.
        mention_labels -- A list of zero or more MentionLabel objects.

    Return:
        A pair of values, the first being a boolean description of whether
        we were successfully able to connect to the database and perform the
        expected queries.

        If the first value is False, the second value will be an exception
        object, describing the database error.

        If the first value is True, the second value is None.
    """
    cursor = psql.cursor()

    for a_mention_label in mention_labels:
        mention = a_mention_label.mention
        update_params = (a_mention_label.is_abusive, mention.id)
        update_query = """
            UPDATE
                mentions
            SET
                is_abusive = %s
            WHERE
                id = %s;
        """

        try:
            cursor.execute(update_query, update_params)
        except psycopg2.Error, err:
            return False, err

    return True, None

def posts_to_scrape(psql, limit=100):
    """Fetches a list of posts that have not yet been scraped for comment data.

    This function currently only returns Facebook and Instragram posts.

    Args:
        psql -- A psycopg2.connection instance.

    Keyword Args:
        limit -- An integer, describing the maximum number of posts that
                 should be returned.

    Returns:
        A pair of objects, the first being a boolean description of whether
        we were successfully able to connect to the database and perform the
        expected queries.

        If the first value is False, the second value will be an exception
        object, describing the database error.

        If the first value is True, the second value will be a list of zero
        or more Post values, describing posts that haven't had their comments
        extracted yet.
    """
    cursor = psql.cursor()
    select_query = """
        SELECT
            p.id,
            p.body,
            a.name,
            n.name
        FROM
            posts AS p
        JOIN
            crawls AS c ON (p.crawl_id = c.id)
        JOIN
            accounts AS a ON (c.account_id = a.id)
        JOIN
            networks AS n ON (a.network_id = n.id)
        WHERE
            p.comments_checked = False AND
            a.network_id IN (1, 2)
        LIMIT
            %s;
    """

    try:
        cursor.execute(select_query, (int(limit),))
        return True, [Post(*row) for row in cursor.fetchall()]
    except ValueError, err:
        return False, Exception("Invalid integer provided as a limit.")
    except psycopg2.Error, err:
        return False, err

def record_comments_for_post(psql, post, comment_labels):
    """Records comments extracted from a post, including whether those comments
    were abusive or not to the account holder.

    Args:
        psql           -- A psycopg2.connection instance.
        post            -- A doxcommentextractor.types.Post object, describing
                          the post the comments were made on.
        comment_labels -- A list of zero or more
                          doxcommentextractor.types.CommentLabel objects,
                          each describing a comment on the given post.

    Return:
        A pair of values, the first being a boolean description of whether
        we were successfully able to connect to the database and perform the
        expected queries.

        If the first value is False, the second value will be an exception
        object, describing the database error.

        If the first value is True, the second value is None.
    """
    cursor = psql.cursor()

    if len(comment_labels) > 0:
        rows_to_insert = [(post.id, cl.is_abusive, cl.comment.text, cl.comment.commenter) for cl in comment_labels]
        values_to_insert = list(itertools.chain.from_iterable(rows_to_insert))
        query_placeholders = ", ".join(("(%s, %s, %s, %s)",) * len(comment_labels))
        insert_query = """
            INSERT INTO
                comments (
                post_id,
                is_abusive,
                text,
                commenter
            ) VALUES {};
        """.format(query_placeholders)

        try:
            cursor.execute(insert_query, values_to_insert)
        except psycopg2.Error, err:
            return False, err

    # And now that we've inserted all the comment values in this transaction,
    # we go back and update the post row, to indicate that we've recorded
    # all the comments on this post.
    update_post_query = """
        UPDATE
            posts AS p
        SET
            comments_checked = True
        WHERE
            p.id = %s;
    """
    try:
        cursor.execute(update_post_query, (post.id,))
    except psycopg2.Error, err:
        psql.rollback()
        return False, err

    psql.commit()
    return True, None
