#!/usr/bin/env python
"""
Tool to repeatedly check the doxxing database for OSN postings that should
have their comments extracted, and then labels each comment as being abusive
or not.
"""

import time
import logging
import argparse
import psycopg2
import doxcommentextractor.models as DM
import doxcommentextractor.comments
import doxcommentextractor.abuse as DA
import doxcommentextractor.reporting
import config

PARSER = argparse.ArgumentParser(description="Extract and label OSN comments "
                                             "in the doxxing database.")
PARSER.add_argument('-l', '--logdir', default=None,
                    help="Path to write logs to.")
PARSER.add_argument('-v', '--verbose', action="store_true",
                    help="Enables debugging / verbose logging.")
PARSER.add_argument('-d', '--dsn', default=None,
                    help="DSN describing how to connect to the postgres "
                         "database. If not provided, the value in config.py "
                         "will be used.")
PARSER.add_argument('-s', '--secs', default=3, type=int,
                    help="Number of seconds to pause before querying the "
                         "database for more posts to extract comments from.")
ARGS = PARSER.parse_args()

doxcommentextractor.reporting.configure_logger(ARGS.logdir, ARGS.verbose)
LOGGER = logging.getLogger("dox-comment-extractor.log")

CONN = psycopg2.connect(ARGS.dsn or config.postgres)

while True:
    time.sleep(ARGS.secs)
    FETCH_SUCCESS, POSTS = DM.posts_to_scrape(CONN)
    if not FETCH_SUCCESS:
        LOGGER.error(u"Error encountered when fetching posts from the database: %(err)s", {"err": str(POSTS)})
        continue

    LOGGER.info(u"Fetched %(num_posts)d posts from the database to check for comments.", {"num_posts": len(POSTS)})

    SUCCESSES = 0
    for A_POST in POSTS:
        EXTRACT_SUCCESS, COMMENTS = doxcommentextractor.comments.extract(A_POST)
        if not EXTRACT_SUCCESS:
            LOG_PARAMS = {"post_id": A_POST.id, "err": str(COMMENTS)}
            LOGGER.error(u"Error encountered when extracting comments from post '%(post_id)d': %(err)s", LOG_PARAMS)
            continue

        LOGGER.info(u"Extracted %(num)d comments from post %(post_id)d.", {"num": len(COMMENTS), "post_id": A_POST.id})
        LABELED_COMMENTS = [DA.label_comment(A_POST, C) for C in COMMENTS]

        WRITE_SUCCESS, ERR = DM.record_comments_for_post(CONN, A_POST, LABELED_COMMENTS)
        if not WRITE_SUCCESS:
            LOG_PARAMS = {"post_id": A_POST.id, "err": str(ERR)}
            LOGGER.error(u"Error encountered when attempting to write comment data for post '%(num)d': %(err)s", LOG_PARAMS)
        else:
            SUCCESSES += 1
    if SUCCESSES > 0:
        LOGGER.info("Successfully recorded comments for %(num)d posts.", {"num": SUCCESSES})

    # Also check if there are any twitter mentions in the database that need
    # labeling.
    MENTIONS_SUCCESS, MENTIONS = DM.fetch_mentions_to_evaluate(CONN)
    if not MENTIONS_SUCCESS:
        LOG_PARAMS = {"err": str(MENTIONS)}
        LOGGER.error(u"Error encountered when fetching mentions to update: %(err)s", LOG_PARAMS)
        continue

    LOG_PARAMS = {"num_mentions": len(MENTIONS)}
    LOGGER.info(u"Fetched %(num_mentions)d mentions to check for abuse.", LOG_PARAMS)

    LABELED_MENTIONS = [DA.label_mention(A_MENTION) for A_MENTION in MENTIONS]
    UPDATE_MENTIONS_SUCCESS, ERR = DM.record_if_mentions_are_abusive(CONN, LABELED_MENTIONS)
    if not UPDATE_MENTIONS_SUCCESS:
        LOG_PARAMS = {"err": str(ERR)}
        LOGGER.error(u"Error encountered when attempting to update mentions records: %(err)s", LOG_PARAMS)
    else:
        LOGGER.info("Successfully updated %(num)d mentions.", {"num": len(LABELED_MENTIONS)})
